from pydantic import BaseModel


class AuthorizeCommand(BaseModel):
    client_id: str
    client_secret: str
    client_scope: str
    redirect_uri: str
    response_type: str
    nonce: str
    state: str
